@@ -38,21 +38,19 @@
 #include "trustdb.h"
 #include "ttyio.h"
 #include "i18n.h"
+#include "status.h"
 
 
 enum cmd_values { aNull = 0,
-    aSym, aStore, aEncr, aPrimegen, aKeygen, aSign, aSignEncr,
-    aPrintMDs, aSignKey, aClearsig, aListPackets, aEditSig,
-    aKMode, aKModeC, aChangePass, aImport, aListTrustDB,
-    aListTrustPath, aExport,
+    aSym, aStore, aEncr, aKeygen, aSign, aSignEncr,
+    aSignKey, aClearsign, aListPackets, aEditSig,
+    aKMode, aKModeC, aChangePass, aImport,
+    aExport,
 aTest };
 
 
 static void set_cmd( enum cmd_values *ret_cmd,
 			enum cmd_values new_cmd );
-static void print_hex( byte *p, size_t n );
-static void print_mds( const char *fname );
-static void do_test(int);
 
 const char *
 strusage( int level )
@@ -104,7 +102,11 @@ static void
 i18n_init(void)
 {
   #ifdef HAVE_LIBINTL
-    setlocale( LC_MESSAGES, "" );
+    #ifdef HAVE_LC_MESSAGES
+       setlocale( LC_MESSAGES, "" );
+    #else
+       setlocale( LC_ALL, "" );
+    #endif
     bindtextdomain( PACKAGE, G10_LOCALEDIR );
     textdomain( PACKAGE );
   #endif
@@ -148,6 +150,9 @@ set_cmd( enum cmd_values *ret_cmd, enum cmd_values new_cmd )
 	cmd = aSignEncr;
     else if( cmd == aKMode && new_cmd == aSym )
 	cmd = aKModeC;
+    else if(	( cmd == aSign	   && new_cmd == aClearsign )
+	     || ( cmd == aClearsign && new_cmd == aSign )  )
+	cmd = aClearsign;
     else {
 	log_error(_("conflicting commands\n"));
 	g10_exit(2);
@@ -187,11 +192,8 @@ main( int argc, char **argv )
     { 'k', NULL      , 0, N_("list keys")},
     { 510, "debug"     ,4|16, N_("set debugging flags")},
     { 511, "debug-all" ,0, N_("enable full debugging")},
- /* { 512 unused */
-    { 513, "gen-prime" , 0, "\r" },
-    { 514, "test"      , 0, "\r" },
+    { 512, "status-fd" ,1, N_("write status info to this fd") },
     { 515, "fingerprint", 0, N_("show the fingerprints")},
-    { 516, "print-mds" , 0, N_("print all message digests")},
     { 517, "secret-keyring" ,2, N_("add this secret keyring to the list")},
     { 518, "options"   , 2, N_("read options from file")},
     { 519, "no-armor",   0, "\r"},
@@ -206,13 +208,13 @@ main( int argc, char **argv )
     { 528, "pubkey-algo", 2 , N_("select default puplic key algorithm")},
     { 529, "digest-algo", 2 , N_("select default message digest algorithm")},
     { 530, "import",      0 , N_("put public keys into the trustdb")},
-    { 531, "list-trustdb",0 , "\r"},
     { 532, "quick-random", 0, "\r"},
-    { 533, "list-trust-path",0, "\r"},
     { 534, "no-comment", 0,   N_("do not write comment packets")},
     { 535, "completes-needed", 1, N_("(default is 1)")},
     { 536, "marginals-needed", 1, N_("(default is 3)")},
     { 537, "export", 0, N_("export all or the given keys") },
+    { 538, "trustdb-name", 2, "\r" },
+    { 539, "clearsign", 0, N_("make a clear text signature") },
 
     {0} };
     ARGPARSE_ARGS pargs;
@@ -235,9 +237,12 @@ main( int argc, char **argv )
     int default_keyring = 1;
     int greeting = 1;
     enum cmd_values cmd = 0;
+    const char *trustdb_name = NULL;
 
-
-    secmem_init( 16384 );
+    /* Please note that we may running SUID(ROOT), so be very CAREFUL
+     * when adding any stuff between here and the call to
+     * secmem_init()  somewhere after the option parsing
+     */
 
     i18n_init();
     opt.compress = -1; /* defaults to standard compress level */
@@ -299,13 +304,13 @@ main( int argc, char **argv )
 		    break;
 	  case 'z': opt.compress = pargs.r.ret_int; break;
 	  case 'a': opt.armor = 1; opt.no_armor=0; break;
+	  case 'd': break; /* it is default */
 	  case 'c': set_cmd( &cmd , aSym); break;
 	  case 'o': opt.outfile = pargs.r.ret_str; break;
 	  case 'e': set_cmd( &cmd, aEncr); break;
-	  case 'b': detached_sig = 1;
-	       /* fall trough */
+	  case 'b': detached_sig = 1; /* fall trough */
 	  case 's': set_cmd( &cmd, aSign );  break;
-	  case 't': set_cmd( &cmd , aClearsig);  break;
+	  case 't': set_cmd( &cmd , aClearsign);  break;
 	  case 'u': /* store the local users */
 	    sl = m_alloc( sizeof *sl + strlen(pargs.r.ret_str));
 	    strcpy(sl->d, pargs.r.ret_str);
@@ -329,11 +334,8 @@ main( int argc, char **argv )
 	  case 509: add_keyring(pargs.r.ret_str); nrings++; break;
 	  case 510: opt.debug |= pargs.r.ret_ulong; break;
 	  case 511: opt.debug = ~0; break;
-	/*  case 512: */
-	  case 513: set_cmd( &cmd, aPrimegen); break;
-	  case 514: set_cmd( &cmd, aTest); break;
+	  case 512: set_status_fd( pargs.r.ret_int ); break;
 	  case 515: opt.fingerprint = 1; break;
-	  case 516: set_cmd( &cmd, aPrintMDs); break;
 	  case 517: add_secret_keyring(pargs.r.ret_str); sec_nrings++; break;
 	  case 518:
 	    /* config files may not be nested (silently ignore them) */
@@ -361,13 +363,13 @@ main( int argc, char **argv )
 	    opt.def_digest_algo = string_to_digest_algo(pargs.r.ret_str);
 	    break;
 	  case 530: set_cmd( &cmd, aImport); break;
-	  case 531: set_cmd( &cmd, aListTrustDB); break;
 	  case 532: quick_random_gen(1); break;
-	  case 533: set_cmd( &cmd, aListTrustPath); break;
 	  case 534: opt.no_comment=1; break;
 	  case 535: opt.completes_needed = pargs.r.ret_int; break;
 	  case 536: opt.marginals_needed = pargs.r.ret_int; break;
 	  case 537: set_cmd( &cmd, aExport); break;
+	  case 538: trustdb_name = pargs.r.ret_str; break;
+	  case 539: set_cmd( &cmd, aClearsign); break;
 	  default : errors++; pargs.err = configfp? 1:2; break;
 	}
     }
@@ -401,6 +403,11 @@ main( int argc, char **argv )
     if( errors )
 	g10_exit(2);
 
+    /* initialize the secure memory. */
+    secmem_init( 16384 );
+    /* Okay, we are now working under our real uid */
+
+    write_status( STATUS_ENTER );
 
     set_debug();
     if( cmd == aKMode || cmd == aKModeC ) { /* kludge to be compatible to pgp */
@@ -451,13 +458,7 @@ main( int argc, char **argv )
 	}
     }
 
-    switch( cmd ) {
-      case aPrimegen:
-      case aPrintMDs:
-	break;
-      case aListTrustDB: rc = init_trustdb( argc? 1:0 ); break;
-      default: rc = init_trustdb(1); break;
-    }
+    rc = init_trustdb(1, trustdb_name );
     if( rc )
 	log_error(_("failed to initialize the TrustDB: %s\n"), g10_errstr(rc));
 
@@ -465,24 +466,24 @@ main( int argc, char **argv )
     switch( cmd ) {
       case aStore: /* only store the file */
 	if( argc > 1 )
-	    usage(1);
+	    wrong_args(_("--store [filename]"));
 	if( (rc = encode_store(fname)) )
-	    log_error("encode_store('%s'): %s\n",
+	    log_error("%s: store failed: %s\n",
 				    fname_print, g10_errstr(rc) );
 	break;
 
       case aSym: /* encrypt the given file only with the symmetric cipher */
 	if( argc > 1 )
-	    usage(1);
+	    wrong_args(_("--symmetric [filename]"));
 	if( (rc = encode_symmetric(fname)) )
-	    log_error("encode_symmetric('%s'): %s\n", fname_print, g10_errstr(rc) );
+	    log_error("%s: symmetric encryption failed: %s\n", fname_print, g10_errstr(rc) );
 	break;
 
       case aEncr: /* encrypt the given file */
 	if( argc > 1 )
-	    usage(1);
+	    wrong_args(_("--encrypt [filename]"));
 	if( (rc = encode_crypt(fname,remusr)) )
-	    log_error("encode_crypt('%s'): %s\n", fname_print, g10_errstr(rc) );
+	    log_error("%s: encryption failed: %s\n", fname_print, g10_errstr(rc) );
 	break;
 
       case aSign: /* sign the given file */
@@ -493,20 +494,20 @@ main( int argc, char **argv )
 	}
 	else {
 	    if( argc > 1 )
-		usage(1);
+		wrong_args(_("--sign [filename]"));
 	    if( argc ) {
 		sl = m_alloc_clear( sizeof *sl + strlen(fname));
 		strcpy(sl->d, fname);
 	    }
 	}
 	if( (rc = sign_file( sl, detached_sig, locusr, 0, NULL, NULL)) )
-	    log_error("sign_file: %s\n", g10_errstr(rc) );
+	    log_error("signing failed: %s\n", g10_errstr(rc) );
 	free_strlist(sl);
 	break;
 
       case aSignEncr: /* sign and encrypt the given file */
 	if( argc > 1 )
-	    usage(1);
+	    wrong_args(_("--sign --encrypt [filename]"));
 	if( argc ) {
 	    sl = m_alloc_clear( sizeof *sl + strlen(fname));
 	    strcpy(sl->d, fname);
@@ -514,33 +515,40 @@ main( int argc, char **argv )
 	else
 	    sl = NULL;
 	if( (rc = sign_file(sl, detached_sig, locusr, 1, remusr, NULL)) )
-	    log_error("sign_file('%s'): %s\n", fname_print, g10_errstr(rc) );
+	    log_error("%s: sign+encrypt failed: %s\n", fname_print, g10_errstr(rc) );
 	free_strlist(sl);
 	break;
 
+      case aClearsign: /* make a clearsig */
+	if( argc > 1 )
+	    wrong_args(_("--clearsign [filename]"));
+	if( (rc = clearsign_file(fname, locusr, NULL)) )
+	    log_error("%s: clearsign failed: %s\n", fname_print, g10_errstr(rc) );
+	break;
+
 
       case aSignKey: /* sign the key given as argument */
 	if( argc != 1 )
-	    usage(1);
+	    wrong_args(_("--sign-key username"));
 	/* note: fname is the user id! */
 	if( (rc = sign_key(fname, locusr)) )
-	    log_error("sign_key('%s'): %s\n", fname_print, g10_errstr(rc) );
+	    log_error("%s: sign key failed: %s\n", fname_print, g10_errstr(rc) );
 	break;
 
       case aEditSig: /* Edit a key signature */
 	if( argc != 1 )
-	    usage(1);
+	    wrong_args(_("--edit-sig username"));
 	/* note: fname is the user id! */
 	if( (rc = edit_keysigs(fname)) )
-	    log_error("edit_keysig('%s'): %s\n", fname_print, g10_errstr(rc) );
+	    log_error("%s: edit signature failed: %s\n", fname_print, g10_errstr(rc) );
 	break;
 
       case aChangePass: /* Chnage the passphrase */
 	if( argc > 1 ) /* no arg: use default, 1 arg use this one */
-	    usage(1);
+	    wrong_args(_("--change-passphrase [username]"));
 	/* note: fname is the user id! */
 	if( (rc = change_passphrase(fname)) )
-	    log_error("change_passphrase('%s'): %s\n", fname_print,
+	    log_error("%s: change passphrase failed: %s\n", fname_print,
 						       g10_errstr(rc) );
 	break;
 
@@ -573,52 +581,18 @@ main( int argc, char **argv )
 	    iobuf_close(a);
 	}
 	else
-	    usage(1);
-	break;
-
-      case aPrimegen:
-	if( argc == 1 ) {
-	    mpi_print( stdout, generate_public_prime( atoi(argv[0]) ), 1);
-	    putchar('\n');
-	}
-	else if( argc == 2 ) {
-	    mpi_print( stdout, generate_elg_prime( atoi(argv[0]),
-						   atoi(argv[1]), NULL ), 1);
-	    putchar('\n');
-	}
-	else if( argc == 3 ) {
-	    MPI g = mpi_alloc(1);
-	    mpi_print( stdout, generate_elg_prime( atoi(argv[0]),
-						   atoi(argv[1]), g ), 1);
-	    printf("\nGenerator: ");
-	    mpi_print( stdout, g, 1 );
-	    putchar('\n');
-	    mpi_free(g);
-	}
-	else
-	    usage(1);
-	break;
-
-      case aPrintMDs:
-	if( !argc )
-	    print_mds(NULL);
-	else {
-	    for(; argc; argc--, argv++ )
-		print_mds(*argv);
-	}
+	    wrong_args(_("-k[v][v][v][c] [keyring]"));
 	break;
 
       case aKeygen: /* generate a key (interactive) */
 	if( argc )
-	    usage(1);
+	    wrong_args(_("--gen-key"));
 	generate_keypair();
 	break;
 
-      case aTest: do_test( argc? atoi(*argv): 0 ); break;
-
       case aImport:
 	if( !argc  )
-	    usage(1);
+	    wrong_args(_("nyi"));
 	for( ; argc; argc--, argv++ ) {
 	    rc = import_pubkeys( *argv );
 	    if( rc )
@@ -635,27 +609,11 @@ main( int argc, char **argv )
 	free_strlist(sl);
 	break;
 
-
-      case aListTrustDB:
-	if( !argc )
-	    list_trustdb(NULL);
-	else {
-	    for( ; argc; argc--, argv++ )
-		list_trustdb( *argv );
-	}
-	break;
-
-      case aListTrustPath:
-	if( argc != 2 )
-	    wrong_args("--list-trust-path [-- -]<maxdepth> <username>");
-	list_trust_path( atoi(*argv), argv[1] );
-	break;
-
       case aListPackets:
 	opt.list_packets=1;
       default:
 	if( argc > 1 )
-	    usage(1);
+	    wrong_args(_("[filename]"));
 	if( !(a = iobuf_open(fname)) )
 	    log_fatal(_("can't open '%s'\n"), fname_print);
 	if( !opt.no_armor ) {
@@ -685,99 +643,9 @@ g10_exit( int rc )
     if( opt.verbose )
 	secmem_dump_stats();
     secmem_term();
-    exit( rc? rc : log_get_errorcount(0)? 2:0 );
+    rc = rc? rc : log_get_errorcount(0)? 2:0;
+    write_status( STATUS_LEAVE );
+    exit(rc );
 }
 
 
-static void
-print_hex( byte *p, size_t n )
-{
-    int i;
-
-    if( n == 20 ) {
-	for(i=0; i < n ; i++, i++, p += 2 ) {
-	    if( i == 10 )
-		putchar(' ');
-	    printf(" %02X%02X", *p, p[1] );
-	}
-    }
-    else {
-	for(i=0; i < n ; i++, p++ ) {
-	    if( i && !(i%8) )
-		putchar(' ');
-	    printf(" %02X", *p );
-	}
-    }
-}
-
-static void
-print_mds( const char *fname )
-{
-    FILE *fp;
-    char buf[1024];
-    size_t n;
-    MD_HANDLE md;
-
-    if( !fname ) {
-	fp = stdin;
-	fname = "[stdin]";
-    }
-    else
-	fp = fopen( fname, "rb" );
-    if( !fp ) {
-	log_error("%s: %s\n", fname, strerror(errno) );
-	return;
-    }
-
-    md = md_open( DIGEST_ALGO_MD5, 0 );
-    md_enable( md, DIGEST_ALGO_RMD160 );
-    md_enable( md, DIGEST_ALGO_SHA1 );
-
-    while( (n=fread( buf, 1, DIM(buf), fp )) )
-	md_write( md, buf, n );
-    if( ferror(fp) )
-	log_error("%s: %s\n", fname, strerror(errno) );
-    else {
-	md_final(md);
-	printf(  "%s:    MD5 =", fname ); print_hex(md_read(md, DIGEST_ALGO_MD5), 16 );
-	printf("\n%s: RMD160 =", fname ); print_hex(md_read(md, DIGEST_ALGO_RMD160), 20 );
-	printf("\n%s:   SHA1 =", fname ); print_hex(md_read(md, DIGEST_ALGO_SHA1), 20 );
-	putchar('\n');
-    }
-
-
-    md_close(md);
-
-    if( fp != stdin )
-	fclose(fp);
-}
-
-
-
-static void
-do_test(int times)
-{
-  #if 0
-    MPI t = mpi_alloc( 50 );
-    MPI m = mpi_alloc( 50 );
-    MPI a = mpi_alloc( 50 );
-    MPI b = mpi_alloc( 50 );
-    MPI p = mpi_alloc( 50 );
-    MPI x = mpi_alloc( 50 );
-
-    /* output = b/(a^x) mod p */
-    log_debug("invm %d times ", times);
-    for( ; times > 0; times -- ) {
-	mpi_fromstr(a, "0xef45678343589854354a4545545454554545455"
-		       "aaaaaaaaaaaaa44444fffdecb33434343443331" );
-	mpi_fromstr(b, "0x8765765589854354a4545545454554545455"
-		       "aaaaaaa466577778decb36666343443331" );
-	mpi_invm( t, a, b );
-	fputc('.', stderr); fflush(stderr);
-    }
-
-
-    m_check(NULL);
-  #endif
-}
-
